import { computed } from 'vue'
import { useRouter, type RouteLocationRaw } from 'vue-router'

/**
 * Bind a modal to a single query parameter in the URL (and vice versa)
 *
 * @param flag query parameter to appear in the `url`, corresponding with `isOpen`
 *
 * This functionality completely independent from the `router` modules.
 */
export function useModal(flag: string) {
    const router = useRouter()
    const query = computed(() => router?.currentRoute.value ? router.currentRoute.value.query : {})

    /**
     * Visibility of this modal
     *
     * Use this function to bind a modal to a query parameter in the URL.
     * Using this helper function will not change the routes.
     *
     * For example, if the modal flag is 'upload':
     *
     * ```vue
     * <template>
     *  <Modal :open="isOpen('upload')">...</Modal>
     * </template>
     * ```
     *
     * Url: `https://funkwhale.audio/?upload`
     *
     * More information on query flags: https://router.vuejs.org/api/type-aliases/LocationQueryValue.html
     *
     * Use `asAttribute` instead to bind a modal to an on/off attribute such as `aria-expanded` or `aria-pressed`
     *
     * @param flag Identifier for the modal
     * @returns a `ref`<boolean>
     */
    const isOpen = computed({
        get() {
            const newValue = flag in query.value && query.value[flag] === null
            // console.log("GET isOpen", flag, "in", query, newValue)
            return newValue
        },
        set(newValue: boolean) {
            // console.log("SET isOpen", query, "+", newValue, ", was", isOpen.value)
            router.push({ query: { ...query.value, [flag]: newValue ? null : undefined }}).catch((err) => {
                throw new Error(`Problem pushing route query: ${err}.`);
            });
            // console.log("DONE SET isOpen", query)
        }
    })

    /**
     *
     * @param flag Identifier for the modal
     * @returns a `RouteLocationRaw` object to use in the `to` prop of a RouterLink component
     */
    const to: RouteLocationRaw = {
        query: { ...query.value, [flag]: null }
    }

    /**
     * Use this function to bind a modal to an on/off attribute such as `aria-expanded` or `aria-pressed`
     *
     * @param flag Identifier for the modal
     * @returns a `ref`<true | undefined>
     */
    const asAttribute = computed(() =>
        isOpen.value || undefined
    )

    /**
     * Toggle the visibility of this modal
     *
     *
     * @param flag Identifier for the modal
     */
    const toggle = () =>
        isOpen.value = !isOpen.value

    return { isOpen, to, asAttribute, toggle }
}
